import arcade
import jacovirt.logger
import logging
from jacovirt.pad.token import Token_generator, Token_trash
from jacovirt.pad.pad import Pad

# Screen title and size
SCREEN_TITLE = "Jaco Pad"
SCREEN_WIDTH = int(1100)
SCREEN_HEIGHT = int(1000)

# Window color
BACKGROUND_COLOR = (133, 100, 100)

# Pad
PAD_X_START = 600
PAD_Y_START = 780
PAD_ROW = 5
PAD_COLUMN = 4
TILE_SIZE = 125
TILE_SPACING = 10


class Window(arcade.Window):
    """Main application class"""

    def __init__(self):
        # Init parent class
        super().__init__(int(SCREEN_WIDTH), int(SCREEN_HEIGHT), SCREEN_TITLE)
        # Set background color
        arcade.set_background_color(BACKGROUND_COLOR)

        # List of token generator
        self.tokens_generator = arcade.SpriteList()

        # List of trash
        self.tokens_trash = arcade.SpriteList()

        # The held token
        self.held_token = None

        # List of tokens
        self.tokens = arcade.SpriteList()

        self.pad = None

    def setup(self):
        """Set up the pad"""
        logging.debug("Set up the pad.")

        # Create a camera
        self.camera = arcade.Camera()

        # Create a token generator
        token_generator = Token_generator("forward", 1)
        token_generator.position = 100, 780
        self.tokens_generator.append(token_generator)

        # Create a right token generator
        token_generator = Token_generator("right", 1)
        token_generator.position = 250, 780
        self.tokens_generator.append(token_generator)

        # Create a left token generator
        token_generator = Token_generator("left", 1)
        token_generator.position = 400, 780
        self.tokens_generator.append(token_generator)

        # Create a trash for token
        trash = Token_trash(150, 150)
        trash.position = 60, 60
        self.tokens_trash.append(trash)

        # Setup the pad
        self.pad = Pad(PAD_X_START, PAD_Y_START, PAD_ROW, PAD_COLUMN, TILE_SIZE, TILE_SPACING)
        
    def on_draw(self):
        """Render the screen"""

        # Clear the screen
        self.clear()

        # Draw the pad
        self.pad.tiles.draw()

        # Draw the token generator
        self.tokens_generator.draw()

        # Draw the trash
        self.tokens_trash.draw()

        # Draw the tokens
        self.tokens.draw()

        self.camera.use()

    def on_key_press(self, symbol, modifiers):
        """Called when the user presses key"""
        logging.debug(f"User press {symbol}, {modifiers}")

        if symbol == arcade.key.R:
            # Clear existing tokens and generators
            self.tokens = arcade.SpriteList()
            self.tokens_generator = arcade.SpriteList()

            self.setup()
            logging.info("Restart")

        if symbol == arcade.key.Q:
            arcade.exit()
        
        if symbol == arcade.key.S:
            instruction = self.pad.get_instructions(self.tokens)
            logging.debug(f"Instruction : {instruction}")

        if symbol == arcade.key.F:
            # When the user hits f. Flip between full and not full screen.
            if self.fullscreen:
                # If in fullscreen, switch to windowed mode
                self.set_fullscreen(False)
                
                # Resize the camera to match the window size
                width, height = self.get_size()
                self.camera.resize(width, height)

                # Reset camera position
                self.camera.move((0, 0))

            else:
                # If not in fullscreen, switch to fullscreen mode

                # Remember the previous window size
                last_width, last_height = self.get_size()

                # Switch to fullscreen
                self.set_fullscreen(True)
                
                # Adjust camera and window size for fullscreen
                width, height = self.get_size()
                self.camera.resize(width, height)

                # Calculate the difference in center position between last window size and new fullscreen size
                self.x_fullscreen_diff = (last_width - width) / 2
                self.y_fullscreen_diff = (last_height - height) / 2

                # Move camera to keep the same center point in the game
                self.camera.move((self.x_fullscreen_diff, self.y_fullscreen_diff))
                
                
    def on_mouse_press(self, x, y, button, key_modifiers):
        """Called when the user presses a mouse button"""
        logging.debug(f"Mouse pressed at {x},{y}")

        # If the game is in fullscreen mode, adjust mouse coordinates
        if self.fullscreen:
            x = x + self.x_fullscreen_diff
            y = y + self.y_fullscreen_diff
            logging.debug(f"Mouse pressed at {x}, {y} in the game window in full screen")
        
        # Get list of tokens we've clicked on
        tokens = arcade.get_sprites_at_point((x, y), self.tokens)

        # Have we clicked on a token?
        if len(tokens) > 0:
            # Might be a stack of tokens, get the top one
            primary_token = tokens[-1]

            # All other tiles, grab the token we are clicking on
            self.held_token = primary_token
            self.held_token_original_position = self.held_token.position

            return
        
        # Get the list of generator we've clicked on
        generators = arcade.get_sprites_at_point((x, y), self.tokens_generator)

        # Have we clicked on a generator ?
        if len(generators) > 0:
            generator = generators[-1]
            logging.debug(f"Cliced on the generator of {generator.type}")
            
            # Create a token and take it
            self.held_token = generator.create_token(*generator.position)
            self.held_token_original_position = self.held_token.position
            self.tokens.append(self.held_token)

    def on_mouse_release(self, x, y, button, modifiers):
        """Called when the user presses a mouse button"""
        logging.debug(f"Mouse relase at {x},{y}")

        if self.held_token is not None:
            # Find the closest trash
            trash, distance = arcade.get_closest_sprite(self.held_token, self.tokens_trash)

            # See if we are in contact with the closest trash, if yes delete the held token
            if arcade.check_for_collision(self.held_token, trash):
                    trash.remove_token(self.held_token)
            
            reset_position = True

            # Find the closest tile
            tile, distance = arcade.get_closest_sprite(self.held_token, self.pad.tiles)

            # See if we are in contact with the closest tile
            if arcade.check_for_collision(self.held_token, tile):
                # Center the token
                self.held_token.position = tile.center_x, tile.center_y

                # Success, don't reset position of tokens
                reset_position = False

            if reset_position:
                # Where-ever we were dropped, it wasn't valid. Reset the each token's position
                # to its original spot.
                self.held_token.position = self.held_token_original_position
                        
            # Remove the token from the hand
            self.held_token = None

    def on_mouse_motion(self, x: float, y: float, dx: float, dy: float):
        """Called when the user moves the mouse"""

        # If we are holding token, move it with the mouse
        if self.held_token != None:
            self.held_token.center_x += dx
            self.held_token.center_y += dy


def main():
    """Main method"""
    window = Window()
    window.setup()
    arcade.run()